#!/usr/bin/env python
# -*- coding: utf-8 -*-
# vim: ai ts=4 sts=4 et sw=4 nu
"""
(c) 2014 - Ronan Delacroix
FTP utils
:author: Ronan Delacroix
"""
import ftplib
import logging
import pyftpdlib.servers
import pyftpdlib.handlers
from . import text

try:
    from pyftpdlib.handlers import TLS_FTPHandler as SFTPHandler
except ImportError:
    from pyftpdlib.handlers import FTPHandler as SFTPHandler  # If that case happens, install pyopenssl


class FTPEventLogger:

    def on_connect(self):
        self.log("Connection received.")

    def on_disconnect(self):
        self.log("Disconnection.")

    def log(self, msg, logfun=None, error=False):
        raise Exception('Sub classes should override this function.')

    def on_login(self, username):
        self.log("User %s logged in." % username)

    def on_login_failed(self, username, password):
        self.log("Login failed with credentials %s/%s." % (username, password))

    def on_logout(self, username):
        self.log("User %s logged out." % username)

    def on_file_sent(self, filepath):
        filepath = text.convert_to_unicode(filepath)
        self.log(u"File %s has been successfully sent (User %s)." % (filepath, self.username))

    def on_file_received(self, filepath):
        filepath = text.convert_to_unicode(filepath)
        self.log(u"User %s has uploaded a new file : %s" % (self.username, filepath))

    def on_incomplete_file_sent(self, filepath):
        filepath = text.convert_to_unicode(filepath)
        self.log("""File %s has been Incompletely sent by user %s...
        Waiting for the user to resume his download.""" % (filepath, self.username), error=True)

    def on_incomplete_file_received(self, filepath):
        filepath = text.convert_to_unicode(filepath)
        self.log("""A new file %s has been uploaded but is incomplete by user %s...
        Waiting for the user to resume his upload.""" % (filepath, self.username), error=True)


class FTPHandler(FTPEventLogger, pyftpdlib.handlers.FTPHandler):

    def log(self, msg, logfun=None, error=False):
        if error:
            logging.error("[FTP] %s" % msg)
        else:
            logging.info("[FTP] %s" % msg)


class SecureFTPHandler(FTPEventLogger, SFTPHandler):

    def log(self, msg, logfun=None, error=False):
        if error:
            logging.error("[FTPS] %s" % msg)
        else:
            logging.info("[FTPS] %s" % msg)


class DummyDictFTPAuthorizer(pyftpdlib.handlers.DummyAuthorizer):
    """
    Dummy Dict FTP Authorizer class.
    Provide authentication through FTP for users stored in a dict.

    About permissions:

        Read permissions:
         - "e" = change directory (CWD command)
         - "l" = list files (LIST, NLST, STAT, MLSD, MLST, SIZE, MDTM commands)
         - "r" = retrieve file from the server (RETR command)

        Write permissions:
         - "a" = append data to an existing file (APPE command)
         - "d" = delete file or directory (DELE, RMD commands)
         - "f" = rename file or directory (RNFR, RNTO commands)
         - "m" = create directory (MKD command)
         - "w" = store a file to the server (STOR, STOU commands)
    """

    def __init__(self, users):
        """
        Constructor
        """
        super(DummyDictFTPAuthorizer, self).__init__()
        for username, user in users.items():
            self.add_user(username,
                          user['password'],
                          user['homedir'],
                          perm=user.get('perm', self.read_perms),
                          msg_login="Hi %s, you're welcome here." % user.get('name', username),
                          msg_quit="Bye %s, hoping you get back soon!" % user.get('name', username)
            )
        self.custom_users = users


def create_server(handler, users, listen_to="", port=21, data_port_range='5500-5700', name="Ronan Python FTP Server", masquerade_ip=None, max_connection=500, max_connection_per_ip=10):
    """
    Runs the FTP Server
    """

    try:
        start, stop = data_port_range.split('-')
        start = int(start)
        stop = int(stop)
    except ValueError:
        raise Exception('Invalid value for data ports')
    else:
        data_port_range = range(start, stop + 1)

    handler.authorizer = DummyDictFTPAuthorizer(users=users)

    handler.banner = "%s. (Advice : Please use UTF-8 encoding and always use Binary mode)" % name
    handler.passive_ports = data_port_range

    if masquerade_ip:
        handler.masquerade_address = masquerade_ip

    #logging.getLogger('pyftpdlib').disabled = True
    pyftpdlib.log.logger = logging.getLogger()  # Replace pyftpd logger by default logger

    # Instantiate FTP server class and listen to 0.0.0.0:21 or whatever is written in the config
    address = (listen_to, port)
    server = pyftpdlib.servers.FTPServer(address, handler)

    # set a limit for connections
    server.max_cons = max_connection
    server.max_cons_per_ip = max_connection_per_ip

    return server


def create_ftp_server(users, listen_to="", port=21, data_port_range='5500-5700',
                      name="FTP Server", masquerade_ip=None, max_connection=500, max_connection_per_ip=10):
    """
        FTP Server implements normal FTP mode.
    """
    handler = FTPHandler
    return create_server(handler, users, listen_to=listen_to, port=port, data_port_range=data_port_range,
                         name=name, masquerade_ip=masquerade_ip, max_connection=max_connection,
                         max_connection_per_ip=max_connection_per_ip)


def create_secure_ftp_server(users, certificate, listen_to="", port=990, data_port_range='5700-5900',
                             name="FTP Server", masquerade_ip=None, max_connection=500, max_connection_per_ip=10):
    """
        FTP Server implements FTPS (FTP over TLS/SSL) mode.
          Note: Connect from client using "FTP over TLS/SSL explicit mode".
    """
    handler = SecureFTPHandler
    handler.certfile = certificate
    return create_server(handler, users, listen_to=listen_to, port=port, data_port_range=data_port_range,
                         name=name, masquerade_ip=masquerade_ip, max_connection=max_connection,
                         max_connection_per_ip=max_connection_per_ip)


"""
Following is only for client FTP using FTPUtil library.
"""


class FTPSession(ftplib.FTP):
    """
    This class is required to connect to a different port with FTPUtil lib.
    """
    def __init__(self, url, user, password, port=21):
        ftplib.FTP.__init__(self)
        self.connect(url, port)
        self.login(user, password)

    """
    Usage :
        import ftputil
        host = ftputil.FTPHost('my.server.url.or.ip', user='root', password='hello', 2121, session_factory=FTPSession)
        names = host.listdir(host.curdir)
        print(names)
    """


#!/usr/bin/env python
# -*- coding: utf-8 -*-
# vim: ai ts=4 sts=4 et sw=4 nu
"""
(c) 2013 - Ronan Delacroix
Settings Utils
:author: Ronan Delacroix
"""
import os
from . import code
from configobj import ConfigObj, ConfigObjError, flatten_errors
from validate import Validator


def from_file(name="config", application_name=None, path_template="/etc/<app_name>"):

    script_folder, app_name = code.get_app_name()
    if not application_name:
        application_name = app_name

    path = path_template.replace('<app_name>', application_name).replace('<name>', name)

    config_path = os.path.join(script_folder, name + '.ini')
    if not os.path.isfile(config_path):
        config_path = os.path.join(path, name + '.ini')
    if not os.path.isfile(config_path):
        print("Settings file %s could not be loaded. Exiting." % config_path)
        exit(1)

    spec_path = os.path.join(script_folder, name + '.spec')
    if not os.path.isfile(spec_path):
        spec_path = os.path.join(path, name + '.spec')
    if not os.path.isfile(spec_path):
        spec_path = None

    conf = {}
    try:
        conf = ConfigObj(
            config_path,
            file_error=True,
            unrepr=True,
            raise_errors=False,
            list_values=True,
            configspec=spec_path
        )
        validator = Validator()
        results = conf.validate(validator, preserve_errors=True)
        if not results:
            #weird return value : True if validated, a dict of errors if not. So can't do a simple "if results:".
            for (section_list, key, _) in flatten_errors(conf, results):
                if key is not None:
                    print('The "%s" key in the section "%s" failed validation' % (key, ', '.join(section_list)))
                else:
                    print('The following section was missing:%s ' % ', '.join(section_list))
            exit(1)

    except (ConfigObjError, IOError) as e:
        print('Could not read config from "%s": %s, %s' % (config_path, e, e.__dict__))
        exit(1)

    config = code.AttributeDict(conf)

    return config



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# vim: ai ts=4 sts=4 et sw=4 nu
"""
(c) 2014 - Ronan Delacroix
SNMP utilities - This requires the pySNMP library
:author: Ronan Delacroix
"""

from pysnmp.entity.rfc3413.oneliner import cmdgen

def snmp_get(oid, ip_address):


    errorIndication, errorStatus, \
                 errorIndex, varBinds = cmdgen.CommandGenerator().getCmd(
        # SNMP v1
        #cmdgen.CommunityData('test-agent', 'public', 0),
        # SNMP v2
        cmdgen.CommunityData('test-agent', 'public'),
        # SNMP v3
        #cmdgen.UsmUserData('test-user', 'authkey1', 'privkey1'),
        cmdgen.UdpTransportTarget((ip_address, 161)),
        # Plain OID
        oid,
        # ((mib-name, mib-symbol), instance-id)
        (('SNMPv2-MIB', 'sysObjectID'), 0)
    )

    if errorIndication:
        raise Exception(errorIndication)
    else:
        if errorStatus:
            raise Exception(errorStatus.prettyPrint() + ' at ' +  errorIndex and varBinds[int(errorIndex)-1] or '?')

    return str(varBinds[0][1])


#!/usr/bin/env python
# -*- coding: utf-8 -*-
# vim: ai ts=4 sts=4 et sw=4 nu
"""
(c) 2013 - Ronan Delacroix
Environment-Based Settings
:author: Ronan Delacroix

# WARNING : THIS MODULE IS ACTUALLY DEPRECATED AND ABANDONNED...

"""
import os
import sys
from . import code as code
from configobj import ConfigObj, ConfigObjError, flatten_errors
from validate import Validator

script_folder = os.path.abspath(os.path.dirname(sys.modules.get('__main__', sys.modules[__name__]).__file__))

environment = "development" #by default
try:
    with open(os.path.join(script_folder, 'environment.txt'), 'r') as f:
        environment = f.readline().strip()
except (ConfigObjError, IOError) as e:
    pass
    #print('Could not read environment from "environment.txt": %s' % e)
    #exit(1)

config_path = os.path.join(script_folder, "settings_%s.ini" % environment)

conf = {}
try:
    conf = ConfigObj(
        config_path,
        file_error=True,
        unrepr=True,
        raise_errors=False,
        list_values=True,
        configspec=os.path.join(script_folder, 'settings.spec')
    )
    validator = Validator()
    results = conf.validate(validator, preserve_errors=True)
    if not results:
        for (section_list, key, _) in flatten_errors(conf, results):
            if key is not None:
                print("The '%s' key in the section '%s' failed validation" % (key, ', '.join(section_list)))
            else:
                print("The following section was missing: %s " % ', '.join(section_list))

except (ConfigObjError, IOError) as e:
    pass
    #print('Environment invalid. Could not read config from "%s": %s, %s' % (config_path, e, e.__dict__))

config = code.AttributeDict(conf)
if config.DEBUG:
    print("Working Folder : " + script_folder)

this_module = sys.modules[__name__]
for k, v in config.iteritems():
    if isinstance(v, dict):
        v = code.AttributeDict(v)
    setattr(this_module, k, v)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# vim: ai ts=4 sts=4 et sw=4 nu
"""
(c) 2014 Ronan Delacroix
Toolbox Lazy Sub-module Loading
:author: Ronan Delacroix
"""
from . import code
code.lazy_load_module(__name__)


#!/usr/bin/env python
# -*- coding: utf-8 -*-
# vim: ai ts=4 sts=4 et sw=4 nu
"""
(c) 2013 - Ronan Delacroix
Network Utils
:author: Ronan Delacroix
"""

import socket


PACKET_SIZE = 4096


def get_local_ip_address(target):
    """
    Get the local ip address to access one specific target.
    """
    ip_adr = ''
    try:
        s = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
        s.connect((target, 8000))
        ip_adr = s.getsockname()[0]
        s.close()
    except:
        pass

    return ip_adr


class SocketClient:
    """
    SocketClient class.
    """

    def __init__(self, host, port, timeout=30, sock=None):
        """
        Creates the socket object, but still does not connect it.
        """
        self.host = host
        self.port = int(port)
        if sock is None:
            self.sock = socket.socket() #socket.AF_INET, socket.SOCK_STREAM)
            self.sock.settimeout(timeout)
        else:
            self.sock = sock

    def connect(self):
        """
        Connect socket to server
        """
        try:
            self.sock.connect((self.host, self.port))
            return self.sock
        except socket.error as ex:
            raise Exception('Unable to connect socket on %s:%s - Error %s' % (self.host, self.port, ex))
        except Exception as ex:
            raise Exception('Unable to connect socket on %s:%s - Error %s' % (self.host, self.port, ex))

    def send_by_packet(self, data):
        """
        Send data by packet on socket
        """
        total_sent = 0
        while total_sent < PACKET_SIZE:
            sent = self.sock.send(data[total_sent:])
            if sent == 0:
                raise RuntimeError("socket connection broken")
            total_sent += sent
        return total_sent

    def sendall(self, data):
        """
        Send all data on socket
        """
        sent = self.sock.sendall(data)
        return sent

    def send(self, data):
        """
        Send data on socket
        """
        return self.sendall(data)

    def receive(self, siz):
        """
        Receive a known length of bytes from a socket
        """
        result = bytearray()
        data = 'x'
        while len(data) > 0:
            data = self.sock.recv(siz - len(result))
            result += data
            if len(result) == siz:
                return result
            if len(result) > siz:
                raise Exception('Received more bytes than expected')
        raise Exception('Error receiving data. %d bytes received'%len(result))

    def close(self):
        self.sock.close()
        self.sock = None




#!/usr/bin/env python
# -*- coding: utf-8 -*-
# vim: ai ts=4 sts=4 et sw=4 nu
"""
:Author: Ronan Delacroix
:Copyright: 2014 Ronan Delacroix


1 - HOW TO WITH FUNCTION :

    def my_service(arg=None):
        print('Hello %s' % arg)

    toolbox.service.launch_function(my_service, arg="Ronan")


2 - HOW TO WITH CLASS :

    class MyService(toolbox.service.Service):
        def run(self):
            print("Hello %s (debug:%s)" % (self.arg, self.debug))

    toolbox.service.launch_service(MyService, arg="Ronan")

"""
import logging
import time
import argparse


class Service:
    """
    Service class.
    Can be launched once or in loop mode.
    """
    def __init__(self, **kwargs):
        logging.debug("Created service %s" % self.service_name)
        for k in kwargs:
            setattr(self, k, kwargs[k])
        self.setup()

    @property
    def service_name(self):
        return self.__class__.__name__.replace('Service', '')

    def setup(self):
        """
        Empty method. Can be overridden.
        Called at object creation.
        """
        logging.debug("Setting up service %s" % self.service_name)

    def destroy(self):
        """
        Empty method. Can be overridden.
        Called at object disposal.
        """
        logging.debug("Destroying service %s" % self.service_name)
        return None

    def run(self):
        """
        Abstract method. Shall be overridden by sub classes.
        """
        raise Exception('run method shall be overridden in sub class.')

    def loop(self, loop_duration=1.0):
        """
        Run the demo suite in a loop.
        """
        logging.info("Running %s in loop mode." % self.service_name)

        res = None
        while True:
            try:
                res = self.run()
                time.sleep(loop_duration)
            except KeyboardInterrupt:
                logging.warning("Keyboard Interrupt during loop, stopping %s service now..." % self.service_name)
                break
            except Exception as e:
                logging.exception("Error in main loop! (%s)" % e)
                raise e
        return res  # returns last result

    def __enter__(self):
        return self

    def __exit__(self, type, value, tb):
        r = self.destroy()
        return r or self


def launch_function(function, description=None, **kwargs):
    if not description:
        description = function.__doc__

    def custom_run(svc):
        function(**kwargs)

    custom_service = type(function.__name__, (Service,), {'run': custom_run})

    return launch_service(custom_service, description=description)


def launch_service(service, description=None, parser_callback=None, **kwargs):
    if not description:
        description = service.__doc__

    parser = argparse.ArgumentParser(description=description)
    group = parser.add_mutually_exclusive_group()
    group.add_argument('-1', '--once', dest='loop', action="store_false",
                       help='Run the service once.', default="loop")
    group.add_argument('-l', '--loop', dest='loop', action="store_true",
                       help='Run the service in loop mode (Default).')
    parser.add_argument("-t", "--loop-duration", dest="loop_duration", type=float,
                        help="Looping duration.", metavar='T', default=1.0)
    parser.add_argument("-d", "--debug", dest="debug", action="store_true",
                        help="Debug mode.", default=False)

    if parser_callback:
        parser_callback(parser)

    args = parser.parse_args()

    # get the data out of args
    data = dict(args._get_kwargs())
    data.pop('loop')
    data.pop('loop_duration')

    kwargs.update(data)  # this will set kwargs["debug"] value at least, plus the arguments added by the parser_callback

    result = None
    svc = service(**kwargs)

    if args.loop:
        result = svc.loop(loop_duration=args.loop_duration)
    else:
        result = svc.run()

    logging.info("Exiting %s service." % svc.service_name)
    svc.destroy()

    return result




from setuptools import setup, find_packages
import os
import re

if os.environ.get('USER', '') == 'vagrant':
    del os.link

requirements = [r.strip() for r in open('requirements.txt').readlines() if not r.startswith('--')]
requirements = [r if ('git+' not in r) else re.sub(r".*egg=(.*)", r"\1", r).strip() for r in requirements]

setup(
    name='tbx',
    version=open('VERSION.txt').read().strip(),
    author='Ronan Delacroix',
    author_email='ronan.delacroix@gmail.com',
    packages=find_packages(where='.', exclude=["*.tests", "*.tests.*", "tests.*", "tests"]),
    package_data={}, #{'mypkg': ['data/*.dat']},
    scripts=[],
    license=open('LICENCE.txt').read().strip(),
    description='Python Toolbox Library. Collection of useful tools and snippets.',
    long_description=open('README.md').read().strip(),
    include_package_data=True,
    install_requires=requirements,
    classifiers=[
        'Topic :: Software Development :: Libraries',
        'Development Status :: 5 - Production/Stable',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Operating System :: MacOS :: MacOS X',
        'Operating System :: POSIX',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3.4',
    ],
)
